package org.protege.editor.owl.ui.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URI;
import java.net.URISyntaxException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.log4j.Logger;
import org.protege.editor.owl.model.entity.AutoIDGenerator;
import org.protege.editor.owl.model.entity.CustomLabelDescriptor;
import org.protege.editor.owl.model.entity.EntityCreationPreferences;
import org.protege.editor.owl.model.entity.IterativeAutoIDGenerator;
import org.protege.editor.owl.model.entity.LabelDescriptor;
import org.protege.editor.owl.model.entity.MatchRendererLabelDescriptor;
import org.protege.editor.owl.model.entity.PseudoRandomAutoIDGenerator;
import org.protege.editor.owl.model.entity.UniqueIdGenerator;
import org.protege.editor.owl.ui.UIHelper;
import org.protege.editor.owl.ui.renderer.OWLRendererPreferences;
import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLAnnotationProperty;
import org.semanticweb.owlapi.vocab.OWLRDFVocabulary;

/**
 * Author: drummond<br>
 * http://www.cs.man.ac.uk/~drummond/<br><br>
 * <p/>
 * The University Of Manchester<br>
 * Bio Health Informatics Group<br>
 * Date: Jul 24, 2008<br><br>
 *
 * see http://protegewiki.stanford.edu/index.php/Protege4NamingAndRendering
 */
public class NewEntitiesPreferencesPanel extends OWLPreferencesPanel implements ActionListener {

	private static final long serialVersionUID = -319986509127880607L;
    private static final String SEP_COLON = ":";
    private static final String SEP_HASH = "#";
    private static final String SEP_SLASH = "/";

	private Logger logger = Logger.getLogger(NewEntitiesPreferencesPanel.class);
    
    // Entity URI panel
    private JRadioButton autoIDURIFragment;
    private JRadioButton colonButton;
    private JRadioButton hashButton;
    private JRadioButton nameAsURIFragment;
    private JRadioButton slashButton;
    private JRadioButton uriBaseActiveOntology;
    private JRadioButton uriBaseSpecifiedURI;
    private JTextField uriDefaultBaseField;

    // Entity Label panel
    private IRI labelAnnotation = null;
    private JButton annotationSelectButton;
    private JCheckBox autoIDLabel;
    private JCheckBox nameAsLabel;
    private JComboBox annotationLangSelector;
	private JLabel langLabel;
    private JLabel uriLabel;
    private JRadioButton customLabelButton;
    private JRadioButton sameAsRendererLabelButton;
    private JTextField annotationURILabel;

    // Auto-generated ID panel
    private JCheckBox saveIterativeIds;
    private JLabel digitCountLabel;
    private JLabel endLabel;
    private JLabel prefixLabel;
    private JLabel startLabel;
    private JLabel suffixLabel;
    private JPanel autoGeneratedIDPanel;
    private JRadioButton iterativeButton;
    private JRadioButton pseudoRandomButton;
    private JRadioButton uniqueIdButton;
    private JSpinner autoIDEnd;
    private JSpinner autoIDStart;
    private JSpinner autoIDDigitCount;
    private JTextField autoIDPrefix;
    private JTextField autoIDSuffix;

    public void initialise() throws Exception {
    	setLayout(new GridLayout(3,1,0,12));
    	add(createEntityURIPanel());
    	add(createEntityLabelPanel());
    	add(createAutoGeneratedIDPanel());
    }
    
    private JPanel createEntityURIPanel() {
    	JPanel panel = new JPanel();
    	panel.setBorder(new TitledBorder("Entity URI"));
    	panel.setLayout(new GridBagLayout());
    	GridBagConstraints c = new GridBagConstraints();
    	
    	// "Start with:" section
    	c.gridx = 0;
    	c.gridy = 0;
    	c.gridwidth = 1;
    	c.gridheight = 1;
    	c.fill = GridBagConstraints.NONE;
    	c.insets = new Insets(12,12,0,12);
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
    	panel.add(new JLabel("Start with:"), c);
    	
    	c.gridx = 1;
    	c.gridy = 0;
    	c.gridwidth = 2;
    	c.insets = new Insets(12,0,0,0);
    	uriBaseActiveOntology = new JRadioButton("Active ontology URI");
    	uriBaseActiveOntology.setSelected(!EntityCreationPreferences.useDefaultBaseIRI());
    	panel.add(uriBaseActiveOntology, c);
    	
    	c.gridx = 1;
    	c.gridy = 1;
    	c.gridwidth = 1;
    	c.insets = new Insets(0,0,0,5);
    	uriBaseSpecifiedURI = new JRadioButton("Specified URI:");
    	uriBaseSpecifiedURI.setSelected(EntityCreationPreferences.useDefaultBaseIRI());
    	panel.add(uriBaseSpecifiedURI, c);
    	
    	c.gridx = 2;
    	c.fill = GridBagConstraints.HORIZONTAL;
    	c.weightx = 1.0;
    	c.insets = new Insets(0,0,0,12);
    	uriDefaultBaseField = new JTextField();
        uriDefaultBaseField.setText(EntityCreationPreferences.getDefaultBaseIRI().toString());
    	panel.add(uriDefaultBaseField, c);
    	
        ButtonGroup group = new ButtonGroup();
        group.add(uriBaseActiveOntology);
        group.add(uriBaseSpecifiedURI);
        
        // "Followed by:" section
        c.gridx = 0;
        c.gridy = 2;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(11,12,0,12);
        c.weightx = 0;
        panel.add(new JLabel("Followed by:"), c);
        
        c.gridx = 1;
        c.insets = new Insets(11,0,0,0);
        hashButton = new JRadioButton(SEP_HASH);
        hashButton.setSelected(EntityCreationPreferences.getDefaultSeparator().equals(SEP_HASH));
        panel.add(hashButton, c);

        c.gridy = 3;
        c.insets = new Insets(0,0,0,0);
        slashButton = new JRadioButton(SEP_SLASH);
        slashButton.setSelected(EntityCreationPreferences.getDefaultSeparator().equals(SEP_SLASH));
        panel.add(slashButton, c);
        
        c.gridy = 4;
        c.insets = new Insets(0,0,0,0);
        colonButton = new JRadioButton(SEP_COLON);
        colonButton.setSelected(EntityCreationPreferences.getDefaultSeparator().equals(SEP_COLON));
        panel.add(colonButton, c);
        
        ButtonGroup group2 = new ButtonGroup();
        group2.add(hashButton);
        group2.add(slashButton);
        group2.add(colonButton);
        
        // "End with:" section
        c.gridx = 0;
        c.gridy = 5;
        c.insets = new Insets(11,12,0,12);
        panel.add(new JLabel("End with:"), c);
        
        c.gridx = 1;
        c.insets = new Insets(11,0,0,0);
    	c.gridwidth = 2;
        nameAsURIFragment = new JRadioButton("User supplied name");
        nameAsURIFragment.setSelected(!EntityCreationPreferences.isFragmentAutoGenerated());
        nameAsURIFragment.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleActionEndWithName();
			}
        });
        panel.add(nameAsURIFragment, c);
        
        c.gridy = 6;
        c.insets = new Insets(0,0,12,0);
        c.weighty = 1.0;
        autoIDURIFragment = new JRadioButton("Auto-generated ID");
        autoIDURIFragment.setSelected(EntityCreationPreferences.isFragmentAutoGenerated());
        autoIDURIFragment.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleActionEndWithID();
			}
        });
        panel.add(autoIDURIFragment, c);
        
        ButtonGroup group3 = new ButtonGroup();
        group3.add(nameAsURIFragment);
        group3.add(autoIDURIFragment);
        
    	return panel;
    }
    
	private JPanel createEntityLabelPanel() {
    	JPanel panel = new JPanel();
    	panel.setBorder(new TitledBorder("Entity Label"));
    	panel.setLayout(new GridBagLayout());
    	GridBagConstraints c = new GridBagConstraints();
    	
    	c.gridx = 0;
    	c.gridy = 0;
    	c.gridwidth = 2;
    	c.gridheight = 1;
    	c.fill = GridBagConstraints.NONE;
    	c.insets = new Insets(12,12,0,0);
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
        nameAsLabel = new JCheckBox("Create label from user supplied name");
        nameAsLabel.setSelected(EntityCreationPreferences.isGenerateNameLabel());
        nameAsLabel.setEnabled(!autoIDURIFragment.isSelected());
        panel.add(nameAsLabel, c);
        
        c.gridy = 1;
        c.insets = new Insets(0,12,11,0);
        autoIDLabel = new JCheckBox("Create label from auto-generated ID");
        autoIDLabel.setSelected(EntityCreationPreferences.isGenerateIDLabel());
        autoIDLabel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				handleActionCreateLabelFromID();
			}
        });
        panel.add(autoIDLabel, c);
        
        final Class<? extends LabelDescriptor> labelDescrCls = EntityCreationPreferences.getLabelDescriptorClass();
        
        c.gridy = 2;
        c.insets = new Insets(0,12,0,0);
		/*sameAsRendererLabelButton = new JRadioButton("Same as label renderer (currently " 
				+ new SimpleIRIShortFormProvider().getShortForm(getFirstRendererLabel()) + ")");*/
		sameAsRendererLabelButton = new JRadioButton("Same as label renderer");
        sameAsRendererLabelButton.setSelected(labelDescrCls.equals(MatchRendererLabelDescriptor.class));
        panel.add(sameAsRendererLabelButton, c);
        
        c.gridy = 3;        
		customLabelButton = new JRadioButton("Custom label");
		customLabelButton.setSelected(labelDescrCls.equals(CustomLabelDescriptor.class));
		panel.add(customLabelButton, c);
		
		ButtonGroup group = new ButtonGroup();
		group.add(sameAsRendererLabelButton);
		group.add(customLabelButton);
		
		c.gridy = 4;
		c.gridwidth = 1;
		c.insets = new Insets(5,30,0,5);
		uriLabel = new JLabel("URI");
		panel.add(uriLabel, c);
		
		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(5,0,0,0);
		c.weightx = 1.0;
		annotationURILabel = new JTextField();
        labelAnnotation = EntityCreationPreferences.getNameLabelIRI();
        if (labelAnnotation == null){
            labelAnnotation = OWLRDFVocabulary.RDFS_LABEL.getIRI();
        }
        annotationURILabel.setText(labelAnnotation.toString());
		annotationURILabel.setEditable(false);
		panel.add(annotationURILabel, c);
    	
        c.gridx = 2;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(5,5,0,12);
        c.weightx = 0;
		annotationSelectButton = new JButton(new AbstractAction("...") {
			private static final long serialVersionUID = 7759812643136092837L;

			public void actionPerformed(ActionEvent event) {
                handleSelectAnnotation();
            }
        });
		panel.add(annotationSelectButton, c);
		
		c.gridx = 0;
		c.gridy = 5;
		c.insets = new Insets(5,30,12,5);
		langLabel = new JLabel("Lang");
		panel.add(langLabel, c);		
		
		c.gridx = 1;
		c.insets = new Insets(5,0,12,0);
        c.weighty = 1.0;
		annotationLangSelector = new UIHelper(getOWLEditorKit()).getLanguageSelector();
        annotationLangSelector.setSelectedItem(EntityCreationPreferences.getNameLabelLang());
		panel.add(annotationLangSelector, c);
		
		return panel;
    }
	
	private JPanel createAutoGeneratedIDPanel() {
		autoGeneratedIDPanel = new JPanel();
		autoGeneratedIDPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
    	autoGeneratedIDPanel.setBorder(new TitledBorder("Auto-generated ID"));
    	
    	JPanel interiorPanel = new JPanel(new BorderLayout(32, 0));
    	interiorPanel.setBorder(BorderFactory.createEmptyBorder(7, 7, 0, 0));
    	
        // Left panel - radio buttons
        JPanel leftPanel = new JPanel();
        leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.PAGE_AXIS));

        pseudoRandomButton = new JRadioButton("Numeric (pseudo random)");
        iterativeButton = new JRadioButton("Numeric (iterative)");
        uniqueIdButton = new JRadioButton("Unique and meaningless");
        
        final Class<? extends AutoIDGenerator> autoIDGenCls = EntityCreationPreferences.getAutoIDGeneratorClass();
        pseudoRandomButton.setSelected(autoIDGenCls.equals(PseudoRandomAutoIDGenerator.class));
        iterativeButton.setSelected(autoIDGenCls.equals(IterativeAutoIDGenerator.class));
        uniqueIdButton.setSelected(autoIDGenCls.equals(UniqueIdGenerator.class));
        
        ButtonGroup group = new ButtonGroup();
        group.add(pseudoRandomButton);
        group.add(iterativeButton);
        group.add(uniqueIdButton);
        
        pseudoRandomButton.addActionListener(this);
        iterativeButton.addActionListener(this);
        uniqueIdButton.addActionListener(this);
        
        leftPanel.add(pseudoRandomButton);
        leftPanel.add(iterativeButton);
        leftPanel.add(uniqueIdButton);
        leftPanel.add(Box.createVerticalGlue());
        
    	// Center panel - random group of components
        JPanel centerPanel = new JPanel();
    	centerPanel.setLayout(new GridBagLayout());
    	GridBagConstraints c = new GridBagConstraints();
        
        // Prefix label
    	c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(0,0,5,0);
        c.anchor = GridBagConstraints.LINE_END;
        prefixLabel = new JLabel("Prefix: "); 
        centerPanel.add(prefixLabel, c);
    	
    	// Prefix text field
        c.gridx = 1;
    	c.gridy = 0;
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
        autoIDPrefix = new JTextField();
        autoIDPrefix.setText(EntityCreationPreferences.getPrefix());
        autoIDPrefix.setColumns(30);
        centerPanel.add(autoIDPrefix, c);
        
        // Suffix label
        c.gridx = 0;
        c.gridy = 1;
        c.anchor = GridBagConstraints.LINE_END;
        suffixLabel = new JLabel("Suffix: ");
        centerPanel.add(suffixLabel, c);
    	
    	// Suffix text field
        c.gridx = 1;
    	c.gridy = 1;
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
    	autoIDSuffix = new JTextField();
    	autoIDSuffix.setText(EntityCreationPreferences.getSuffix());
    	autoIDSuffix.setColumns(30);
    	centerPanel.add(autoIDSuffix, c);
    	
        // Digit count label
    	c.gridx = 0;
        c.gridy = 2;
        c.anchor = GridBagConstraints.LINE_END;
        digitCountLabel = new JLabel("Digit count: ");
        centerPanel.add(digitCountLabel, c);
        
        // Digit count spinner
        c.gridx = 1;
        c.gridy = 2;
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.weightx = 0.5;
        autoIDDigitCount = new JSpinner(new SpinnerNumberModel(6, 0, 255, 1));
        autoIDDigitCount.setValue(EntityCreationPreferences.getAutoIDDigitCount());
        autoIDDigitCount.setPreferredSize(new Dimension(100, 20));
        centerPanel.add(autoIDDigitCount, c);
        
        // Start label
        c.gridx = 0;
        c.gridy = 3;
        c.weightx = 0;
        c.anchor = GridBagConstraints.LINE_END;
        startLabel = new JLabel("Start: ");
        startLabel.setEnabled(iterativeButton.isSelected());
        centerPanel.add(startLabel, c);
        
        // Start spinner
        c.gridx = 1;
        c.gridy = 3;
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
    	autoIDStart = new JSpinner(new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1));
        autoIDStart.setPreferredSize(new Dimension(100, 20));
        autoIDStart.setValue(EntityCreationPreferences.getAutoIDStart());
        autoIDStart.setEnabled(iterativeButton.isSelected());
        autoIDStart.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent event) {
                if ((Integer)autoIDEnd.getValue() != -1 && (Integer)autoIDEnd.getValue() <= (Integer)autoIDStart.getValue()) {
                    autoIDEnd.setValue(autoIDStart.getValue());
                }
            }
        });
        centerPanel.add(autoIDStart, c);
        
        // End label
        c.gridx = 0;
        c.gridy = 4;
        c.anchor = GridBagConstraints.LINE_END;
        endLabel = new JLabel("End: ");
        endLabel.setEnabled(iterativeButton.isSelected());
        centerPanel.add(endLabel, c);
        
        // End spinner
        c.gridx = 1;
        c.gridy = 4;
    	c.anchor = GridBagConstraints.FIRST_LINE_START;
        autoIDEnd = new JSpinner(new SpinnerNumberModel(-1, -1, Integer.MAX_VALUE, 1));
        autoIDEnd.setPreferredSize(new Dimension(100, 20));
        autoIDEnd.setValue(EntityCreationPreferences.getAutoIDEnd());
        autoIDEnd.setEnabled(iterativeButton.isSelected());
        autoIDEnd.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent event) {
                if ((Integer)autoIDEnd.getValue() != -1 && (Integer)autoIDEnd.getValue() <= (Integer)autoIDStart.getValue()) {
                    autoIDStart.setValue(autoIDEnd.getValue());
                }
            }
        });
        centerPanel.add(autoIDEnd, c);
        
        // Remember last ID checkbox
        c.gridx = 1;
        c.gridy = 5;
        c.gridwidth = 2;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weighty = 1.0;
        saveIterativeIds = new JCheckBox("Remember last ID between Prot\u00E9g\u00E9 sessions");
        saveIterativeIds.setSelected(EntityCreationPreferences.getSaveAutoIDStart());
        saveIterativeIds.setEnabled(iterativeButton.isSelected());
        centerPanel.add(saveIterativeIds, c);
        
        // Dummy label for spacing purposes
        c.gridx = 2;
        c.gridy = 0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        c.weighty = 0;
        centerPanel.add(new JLabel(""));

        if ((autoIDURIFragment.isSelected()) || (autoIDLabel.isSelected() && autoIDLabel.isEnabled())) {
        	enableAutoGeneratedIDPanel(true);
        } else {
        	enableAutoGeneratedIDPanel(false);
        }
        
        interiorPanel.add(leftPanel, BorderLayout.LINE_START);
        interiorPanel.add(centerPanel, BorderLayout.CENTER);
        autoGeneratedIDPanel.add(interiorPanel);
    	return autoGeneratedIDPanel;
	}
	
	private void enableAutoGeneratedIDPanel(boolean b) {
	    autoIDDigitCount.setEnabled(b);
	    autoIDEnd.setEnabled(b);
	    autoIDPrefix.setEnabled(b);
	    autoIDStart.setEnabled(b);
	    autoIDSuffix.setEnabled(b);
	    autoGeneratedIDPanel.setEnabled(b);
	    digitCountLabel.setEnabled(b);
	    endLabel.setEnabled(b);
		iterativeButton.setEnabled(b);
		prefixLabel.setEnabled(b);
		pseudoRandomButton.setEnabled(b);
	    saveIterativeIds.setEnabled(b);
	    startLabel.setEnabled(b);
	    suffixLabel.setEnabled(b);
		uniqueIdButton.setEnabled(b);
		
		enableNumericIterativeOptions((iterativeButton.isSelected()) && (iterativeButton.isEnabled()));
	}
	
	private void enableNumericIterativeOptions(boolean b) {
		startLabel.setEnabled(b);
		autoIDStart.setEnabled(b);
		endLabel.setEnabled(b);
		autoIDEnd.setEnabled(b);
		saveIterativeIds.setEnabled(b);
	}
	
	public void actionPerformed(ActionEvent e) {
		Object object = e.getSource();
		
		if (object == iterativeButton) { // "Numeric (iterative)"
			enableNumericIterativeOptions(true);
		} else if ((object == pseudoRandomButton) || (object == uniqueIdButton)) { // "Numeric (pseudo random)", "Unique and meaningless" 
			enableNumericIterativeOptions(false);
		}
	}
	
    public void applyChanges() {
        EntityCreationPreferences.setUseDefaultBaseIRI(uriBaseSpecifiedURI.isSelected());
        try {
            IRI defaultBase = IRI.create(new URI(uriDefaultBaseField.getText()));
            EntityCreationPreferences.setDefaultBaseIRI(defaultBase);
        }
        catch (URISyntaxException e) {
            logger.error("Ignoring invalid base URI (" + uriDefaultBaseField.getText() + ")");
        }

        if (hashButton.isSelected()){
            EntityCreationPreferences.setDefaultSeparator(SEP_HASH);
        }
        else if (slashButton.isSelected()){
            EntityCreationPreferences.setDefaultSeparator(SEP_SLASH);
        }
        else if (colonButton.isSelected()){
            EntityCreationPreferences.setDefaultSeparator(SEP_COLON);
        }

        EntityCreationPreferences.setFragmentAutoGenerated(autoIDURIFragment.isSelected());

        EntityCreationPreferences.setGenerateNameLabel(nameAsLabel.isSelected());
        EntityCreationPreferences.setGenerateIDLabel(autoIDLabel.isSelected());

        if (sameAsRendererLabelButton.isSelected()){
            EntityCreationPreferences.setLabelDescriptorClass(MatchRendererLabelDescriptor.class);
        }
        if (customLabelButton.isSelected()){
            EntityCreationPreferences.setLabelDescriptorClass(CustomLabelDescriptor.class);
        }

        EntityCreationPreferences.setNameLabelIRI(IRI.create(annotationURILabel.getText()));
        Object lang = annotationLangSelector.getSelectedItem();
        if (lang != null && !lang.equals("")){
            EntityCreationPreferences.setNameLabelLang((String)lang);
        }
        else{
            EntityCreationPreferences.setNameLabelLang(null);
        }

        if (pseudoRandomButton.isSelected()){
            EntityCreationPreferences.setAutoIDGeneratorClass(PseudoRandomAutoIDGenerator.class);
        }
        if (iterativeButton.isSelected()){
            EntityCreationPreferences.setAutoIDGeneratorClass(IterativeAutoIDGenerator.class);
        }
        if (uniqueIdButton.isSelected()) {
        	EntityCreationPreferences.setAutoIDGeneratorClass(UniqueIdGenerator.class);
        }

        EntityCreationPreferences.setAutoIDStart((Integer)autoIDStart.getValue());
        EntityCreationPreferences.setAutoIDEnd((Integer)autoIDEnd.getValue());

        EntityCreationPreferences.setAutoIDDigitCount((Integer)autoIDDigitCount.getValue());
        EntityCreationPreferences.setPrefix(autoIDPrefix.getText());
        EntityCreationPreferences.setSuffix(autoIDSuffix.getText());
        EntityCreationPreferences.setSaveAutoIDStart(saveIterativeIds.isSelected());
    }
    
    private void handleActionCreateLabelFromID() {
    	if (autoIDLabel.isSelected()) {
    		enableAutoGeneratedIDPanel(true);
    	} else if (!autoIDURIFragment.isSelected()) {
    		enableAutoGeneratedIDPanel(false);
    	}
    }
    
    private void handleActionEndWithID() {
    	boolean selected = autoIDURIFragment.isSelected();
    	enableAutoGeneratedIDPanel(selected);
    	nameAsLabel.setEnabled(!selected);
    }
    
    private void handleActionEndWithName() {
    	boolean selected = nameAsURIFragment.isSelected();
    	if (selected && !autoIDLabel.isSelected()) {
    		enableAutoGeneratedIDPanel(false);
    	}
    	nameAsLabel.setEnabled(selected);
    }

    protected void handleSelectAnnotation() {
        OWLAnnotationProperty prop = new UIHelper(getOWLEditorKit()).pickAnnotationProperty();
        if (prop != null){
            labelAnnotation = prop.getIRI();
            annotationURILabel.setText(labelAnnotation.toString());
        }
    }
    
    public IRI getFirstRendererLabel() {
        final java.util.List<IRI> iris = OWLRendererPreferences.getInstance().getAnnotationIRIs();
        if (!iris.isEmpty()){
            return iris.get(0);
        }
        return OWLRDFVocabulary.RDFS_LABEL.getIRI();
    }

    public void dispose() throws Exception {
    }
}
